/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl;

import com.topcoder.document.index.persistence.impl.db.BaseDocumentIndexerPersistenceTest;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.util.Locale;


/**
 * This class contains the unit tests for {@link Utils}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class UtilsTest extends BaseDocumentIndexerPersistenceTest {
    /**
     * This is the name of a valid config property that exists in the current setup.
     */
    private static final String VALID_PROPERTY_NAME = "ConnectionName";

    /**
     * This method tests {@link Utils#lookupOptionalStringFromConfigManager(String, String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupStringFromConfigManager() throws Exception {
        assertEquals("simple", Utils.lookupOptionalStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE,
            UtilsTest.VALID_PROPERTY_NAME));
        assertNull(Utils.lookupOptionalStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE,
            "unknownProperty"));
        assertNull(Utils.lookupOptionalStringFromConfigManager("UnknownNamespace",
            UtilsTest.VALID_PROPERTY_NAME));
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupOptionalStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> namespace argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupStringFromConfigManagerFailureNullNamespace() throws Exception {
        try {
            Utils.lookupOptionalStringFromConfigManager(null, UtilsTest.VALID_PROPERTY_NAME);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupOptionalStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> property argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupStringFromConfigManagerFailureNullProperty() throws Exception {
        try {
            Utils.lookupOptionalStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupOptionalStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: empty String property argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupStringFromConfigManagerFailureEmptyProperty() throws Exception {
        try {
            Utils.lookupOptionalStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE, "   ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupOptionalStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: empty String namespace argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupStringFromConfigManagerFailureEmptyNamespace() throws Exception {
        try {
            Utils.lookupOptionalStringFromConfigManager("  ", UtilsTest.VALID_PROPERTY_NAME);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }

    }

    /**
     * This method tests {@link Utils#lookupValidStringFromConfigManager(String, String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupValidStringFromConfigManager() throws Exception {
        assertEquals("simple", Utils.lookupValidStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE,
            UtilsTest.VALID_PROPERTY_NAME));
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupValidStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> namespace argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupValidStringFromConfigManagerFailureNullNamespace() throws Exception {
        try {
            Utils.lookupValidStringFromConfigManager(null, UtilsTest.VALID_PROPERTY_NAME);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupValidStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> property argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupValidStringFromConfigManagerFailureNullProperty() throws Exception {
        try {
            Utils.lookupValidStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupValidStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: empty String property argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupValidStringFromConfigManagerFailureEmptyProperty() throws Exception {
        try {
            Utils.lookupValidStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE, "   ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupValidStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: empty String namespace argument
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testLookupValidStringFromConfigManagerFailureEmptyNamespace() throws Exception {
        try {
            Utils.lookupValidStringFromConfigManager("  ", UtilsTest.VALID_PROPERTY_NAME);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupValidStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given namespace is unknown in {@link com.topcoder.util.config.ConfigManager}
     */
    public void testLookupValidStringFromConfigManagerFailureUnknownNamespace() {
        try {
            Utils.lookupValidStringFromConfigManager("unknown", UtilsTest.VALID_PROPERTY_NAME);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupValidStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given property is unknown in {@link com.topcoder.util.config.ConfigManager}
     */
    public void testLookupValidStringFromConfigManagerFailureUnknownProperty() {
        try {
            Utils.lookupValidStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE, "blah");
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#lookupValidStringFromConfigManager(String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given property is unknown in {@link com.topcoder.util.config.ConfigManager}
     */
    public void testLookupValidStringFromConfigManagerFailurePropertyIsEmptyString() {
        try {
            Utils.lookupValidStringFromConfigManager(SIMPLE_DB_CONFIGURATION_NAMESPACE, "emptyProperty");
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link Utils#serializeObject(Object)} and {@link Utils#deSerializeObject(byte[])}for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testSerializeDeserialize() throws Exception {
        final Object source = getClass();
        final byte[] bytes = Utils.serializeObject(source);
        final Object copy = Utils.deSerializeObject(bytes);
        assertEquals("Serialization mismatch", source, copy);
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#serializeObject(Object)}.
     * <p/>
     * <b>Failure reason</b>: the given argument is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testSerializeFailureNull() throws Exception {
        try {
            Utils.serializeObject(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#deSerializeObject(byte[])}.
     * <p/>
     * <b>Failure reason</b>: the given argument is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDeSerializeFailureNull() throws Exception {
        try {
            Utils.deSerializeObject(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link Utils#createIdString(WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateId() throws Exception {
        final WordSourceId sourceId = new WordSourceId(getClass(), getClass().getName(), new String[]{"\r", ", "},
            Locale.getDefault());
        final String idString = Utils.createIdString(sourceId);
        assertNotNull(idString);
        assertEquals("Id generation changed", "ea4036af014d8de25e329169d0869e4c", idString);
    }

    /**
     * This method tests the correct failure behavior of {@link Utils#createIdString(WordSourceId)}.
     * <p/>
     * <b>Failure reason</b>: the given argument is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCreateIdStringFailureNull() throws Exception {
        try {
            Utils.createIdString(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

}
